package atomfeed

import (
	"errors"
	"fmt"
)

type Category struct {
	*CommonAttributes
	Content Content `xml:"content"` // should this even exist in here?
	Term    string  `xml:"term,attr"`
	Scheme  URI     `xml:"scheme,attr,omitempty"`
	Label   string  `xml:"label,attr,omitempty"`
}

func NewCategory(term string) (*Category, error) {
	content, err := NewContent(InlineText, "", "")
	if err != nil {
		return nil, fmt.Errorf("error creating content element: %v", err)
	}

	return &Category{Term: term, Content: content}, nil
}

func (c *Category) Check() error {
	if c.Term == "" {
		return errors.New("term attribute of category empty")
	}

	if c.Scheme != "" {
		if !isValidURI(c.Scheme) {
			return fmt.Errorf("scheme attribute of category %v not correctly formatted", c.Scheme)
		}
	}

	if c.Content == nil {
		return errors.New("no content element of category")
	} else {
		if err := c.Content.Check(); err != nil {
			return fmt.Errorf("content element of category: %v", err)
		}
	}

	return nil
}
